/**
 * \file CostManager.hpp
 *
 * Header file to declare cost manager functions
 *
 * \author Redouane Boumghar, Hung Cao
 *
 * \ingroup lgl
 *
 **/

#ifndef LGL_COST_MANAGER_HPP
#define LGL_COST_MANAGER_HPP

#include <lgl/GeoData.hpp>
#include <lgl/Quadtree.hpp>
#include <lgl/PositionManager.hpp>

#include <lgl/NavHeuDistances.hpp>
#include <lgl/NavGNode.hpp>
#include <lgl/Attributes.hpp>

namespace jafar { //namespace jafar
	namespace lgl { //namespace lgl

		/**
		 * Cost manager take care of the cost computing between two given ADJACENT positions in a graph
		 * and conversion between the cost and the time taken by the robot to travel this edge.
		 *
		 * @ingroup lgl
		 */
		class CostManager {/*{{{*/
								
			public:

				/** Different distance types */
				enum DIST_TYPE {EUCLIDEAN=0, MANHATTAN};
				
				CostManager(GeoData* _geodata, Quadtree<double,GeoData>* _quadtree, PositionManager* _posManager, DIST_TYPE _distType=EUCLIDEAN);
				~CostManager();
				
				/** Set the way distances will be calculated
				 *
				 * @param _distType New distance type to use.
				 *
				 */
				void setDistType(DIST_TYPE _distType);

				/** Set the obstacle probability limit.
				 * Above this limit and from it, weights will be considered as maximum (infinity)
				 *
				 * @param pol obstacle probability limit
				 */
				void setObstacleLimit(double pol);

				/** New navigation cost function 
				 *
				 * @param locClusterRootIndex localisation of the edge cluster
				 *   to be able to get attributes from it (thanks to the
				 *   positionManager)
				 */
				bool weight(RasterCellIndex locClusterRootIndex, const NavLocation& loc1, const NavLocation& loc2, double& weight);

				/** Effective Weighting function */
				bool weight(const NavLocation& loc1, const NavLocation& loc2, double& weight);

				/** Weighting function adding the effect of the resulting position */
				bool weight(const NavLocation& loc1, const NavLocation& loc2, double costAtLoc2, double& weight);

			protected:
				
				/** GeoData with GDAL dataset */
				GeoData* geodata;

				/** Position manager holding navigation location */
				PositionManager* posManager;

				/** Quadtree holding the world decomposition structure */
				Quadtree<double,GeoData>* quadtree;

				/** Used distance for weight calculation
				 *
				 * @see enum DIST_TYPE
				 */
				DIST_TYPE distType;

				/** Obstacle probability limit */
				double obstacleLimit;

				
		}; /*}}}*/
	}
}

#endif /* LGL_COST_MANAGER_HPP */
