/**
 * \file Decomposer.hpp
 *
 * Header file to declare and define some terrain decomposition functions
 *
 * \date 2009-09-04
 * \author Redouane Boumghar, Hung Cao
 *
 * \ingroup lgl
 *
 **/

#ifndef LGL_TERRAIN_DECOMPOSER_HPP
#define LGL_TERRAIN_DECOMPOSER_HPP

#include <vector>

#include <lgl/Raster.hpp>
#include <lgl/Decomp.hpp>

namespace jafar { // namespace jafar
	namespace lgl { // namespace lgl
		
		/**
		 * Abstract class that all terrain decomposers (bin tree, quadtree or any rectangular clustering)
		 */
		template <
			typename T,
			class RasterT
			>
		class TDecomposer  {
				
			protected:
				
				/** Pointer to the raster object of type Raster
				 *
				 * It represents the data of the terrain over which the quadtree is built.
				 * This points generally to an external object. (Do not delete)
				 */
				RasterT* raster;
				
				/** The decomposition matrix structure
				 *
				 * It represents in a raster/matrix way the geometric rectangular decomposition
				 */
				Decomp* decomp;
				
			public:
				
				TDecomposer(RasterT* _raster);
				
				virtual ~TDecomposer();

				RasterT* getRaster() const;
				Decomp* getDecomp() const;

				// Abstract methods of the decomposer
				// dont forget =0 in virtual method
				// the decomposer should be able to return cost (floating number) of a cluster
				virtual T getData(const RasterRect& rect)const=0;
				virtual T getData(const RasterCellIndex& cluster)const=0;
				
				// get the cluster for a given cell
				virtual bool getClusterForCell(const RasterCellIndex& cell, RasterRect& cluster) const=0;
				virtual bool getClusterForCell(const RasterCellIndex& cell, RasterCellIndex& clusterRoot) const=0;
				
				// get all adjacents clusters of a given cluster
				virtual bool getAdjacentClusters( const RasterCellIndex& root_cell,
				                                  Decomp::SIDE side,
												  std::list<RasterCellIndex>& clusters)const=0;		

		};

			template <typename T, class RasterT>
			TDecomposer<T,RasterT>::TDecomposer(RasterT* _raster):
				raster(_raster)
			{
				if (raster == NULL)
				 return;
				
				// create the geometric decomposition data structure
				decomp = new Decomp(raster);
				
			}

			template <typename T, class RasterT>
			TDecomposer<T,RasterT>::~TDecomposer() { }
			
			template <typename T, class RasterT>
			RasterT*  TDecomposer<T,RasterT>::getRaster() const {
				return raster;
			}
			
			
			template <typename T, class RasterT>
			Decomp* TDecomposer<T,RasterT>::getDecomp() const {
				return decomp;
			}


	}
}

#endif // LGL_TERRAIN_DECOMPOSER_HPP
