/**
 * \file Location.hpp
 *
 * Header file for navigation structure declaration and definition
 * 
 *
 * \author Redouane Boumghar, Hung Cao
 *
 * \ingroup lgl
 *
 **/

#ifndef LGL_LOCATION_HPP
#define LGL_LOCATION_HPP

#include <iostream>


namespace jafar {
	namespace lgl {

		struct NavLocation {

			/** Position indexes.
			 *
			 * The begining of the first left top corner of the left top cell/pixel is (0,0)
			 * the middle of the first x edge is position (1,0)
			 * etc.
			 * This way we get indexes for all possible position 
			 */
			int pos_x; 
			int pos_y;
			int pos_z;

			// Triplet for UTM position
			double utm_x;
			double utm_y;
			double utm_z;

			// Default constructor
			NavLocation():pos_x(0),pos_y(0),pos_z(0),utm_x(0.0),utm_y(0.0),utm_z(0.0) {}
			// (X,Y) constructor
			NavLocation(int in_xxx, int in_yyy):
			pos_x(in_xxx),pos_y(in_yyy),pos_z(0),utm_x(0.0),utm_y(0.0),utm_z(0.0) {}
			// (X,Y,Z) constructor
			NavLocation(int in_xxx, int in_yyy, int in_zzz):
			pos_x(in_xxx),pos_y(in_yyy),pos_z(in_zzz),utm_x(0.0),utm_y(0.0),utm_z(0.0) {}

			void setPos(int in_xxx, int in_yyy) {pos_x=in_xxx;pos_y=in_yyy;}
			void setUTMPos(double in_xxx, double in_yyy, double _z) {
				utm_x = in_xxx;
				utm_y = in_yyy;
				utm_z = _z;
			}

			/** Set UTM Position by calculating the indexes offsets. UTM Origin
			 * (left top image corner) and cell Resolutions must be provided */
			void updateUTMPos(
			    double utmXRoot,
			    double utmYRoot,
			    double cellXRes,
			    double cellYRes
				)
			{
				utm_x = utmXRoot + ((double)(pos_x)/2.0) * cellXRes;
				// using minus sign for pos_y since pos_y is negative to suite image frames.
				utm_y = utmYRoot + ((double)(pos_y)/2.0) * cellYRes;
				utm_z = 0.0;
			}

			int x() const {return pos_x;}
			int y() const {return pos_y;}

			bool operator==(const  NavLocation &rhs) const
			{
				return (pos_x == rhs.pos_x) && (pos_y == rhs.pos_y);
			}
			
			bool operator!=(const  NavLocation &rhs) const
			{
				return (pos_x != rhs.pos_x) || (pos_y != rhs.pos_y);
			}

			NavLocation& operator=(const NavLocation& loc) {
				pos_x=loc.pos_x;
				pos_y=loc.pos_y;
				utm_x = loc.utm_x;
				utm_y = loc.utm_y;
				utm_z = loc.utm_z;
				return *this;
			}

		};

		std::ostream& operator<<(std::ostream& out, const NavLocation &loc);

	}
}

#endif /* LGL_LOCATION_HPP */
