/**
 * \file NavGNode.hpp
 *
 * Header file for the navigation graph node structure
 * 
 *
 * \author Redouane Boumghar, Hung Cao
 *
 * \ingroup lgl
 *
 **/

#ifndef LGL_ENVIRONMENT_GRAPH_NODE_HPP
#define LGL_ENVIRONMENT_GRAPH_NODE_HPP

#include <lgl/Location.hpp>
#include <iostream>

namespace jafar {
	namespace lgl {
		
		/**
		 * NavGNode is the class for representing each node in the navigation graph.
		 * Each node corresponds to a position in the environment. 
		 */
		struct NavGNode {
 		///{{{ 
  			/** Nodes position held in a navigation location */
			NavLocation pos;
			
			/** Goal Reaching Potential
			 *
			 * At each node visited during the path tracing algorithm
			 * we can evaluate the goal reaching potential
			 *
			 */
			double grp;

			/** Is true if the potential has been set at least once */
			bool hasPotential;
			
			/**
			 * @param x X position of the node 
			 * @param y Y position of the node 
			 * @note (x,y) this is not the raster position but the true int position inside the environment
			 */
			NavGNode(int _x=0, int _y=0):pos(_x,_y), grp(0.0), hasPotential(false)
			{ }
			
			NavGNode(const  NavLocation& _loc):
				pos(_loc.pos_x,_loc.pos_y),
				grp(0.0),
				hasPotential(false)
			{
				pos.setUTMPos(_loc.utm_x, _loc.utm_y, _loc.utm_z);
			}
			
			~NavGNode() { }

			/** set this node potential */
			void setPotential(double val) {//{{{
				if (hasPotential)
				{
					grp = grp + val;
				} else {
					grp = val;
					hasPotential = true;
				}
			}//}}}

			/** Equality boolean operator */
 			bool operator==(const NavGNode &rhs) const
			{ //{{{
				return ((pos.pos_x == rhs.pos.pos_x) 
				     && (pos.pos_y == rhs.pos.pos_y)
				     && (pos.pos_z == rhs.pos.pos_z));
			} //}}}

			/** Operator < using lexical order */
			bool operator< (const NavGNode &rhs) const // need to be const (for std:set<#>)
			{///{{{ 
			  if ( pos.pos_x < rhs.pos.pos_x ) return (true ); 
			  if ( pos.pos_x > rhs.pos.pos_x ) return (false); 
			  if ( pos.pos_y < rhs.pos.pos_y ) return (true );
			  if ( pos.pos_y > rhs.pos.pos_y ) return (false);
			  if ( pos.pos_y > rhs.pos.pos_y ) return (true );
			  else return (false) ;
			}///}}} 
							
			/** Operator >  using lexical order
			 */
			bool operator> (const NavGNode &rhs) const // need to be const (for std:set<#>)
			{///{{{
			  if ( pos.pos_x > rhs.pos.pos_x ) return (true ); 
			  if ( pos.pos_x < rhs.pos.pos_x ) return (false); 
			  if ( pos.pos_y > rhs.pos.pos_y ) return (true );
			  if ( pos.pos_y < rhs.pos.pos_y ) return (false);
			  if ( pos.pos_y > rhs.pos.pos_y ) return (true );
			  else return (false) ;
			}///}}}  

		};///}}}

	
		/* output stream for NavGNode */
		std::ostream& operator<<(std::ostream& out, NavGNode const &rhs);

	}
}

#endif /* LGL_ENVIRONMENT_GRAPH_NODE_HPP */
