/**
 * \file PositionManager.hpp
 *
 * Header file for the position manager.
 *
 * \date 2009-09-04
 * \author Redouane Boumghar, Hung Cao
 *
 * \ingroup lgl
 *
 **/

#ifndef LGL_POSITION_MANAGER_HPP
#define LGL_POSITION_MANAGER_HPP

#include <vector>
#include <ostream>

#include <lgl/FlexGrid.hpp>

#include <lgl/Location.hpp>
#include <lgl/GeoData.hpp>
#include <lgl/Raster.hpp>
#include <lgl/Quadtree.hpp>
#include <lgl/Decomp.hpp>

namespace jafar {
  namespace lgl {

	 struct PosFeature {
		bool active;
		
		PosFeature(bool _active=true):active(_active) {}
	 };
	 
	 /**
	  * Class that take care position management its algorithms reason 
	  * over position instead of raster cells or clusters
	  */	 
	 class PositionManager {
		
		typedef FlexGrid<PosFeature> PosMap;
		
	 protected :
		
		/** Main map borders, main global rectangle, everything-container 
		 * in terms of position indexes.
		 */
		RasterRect rect;
		PosMap pos;
		GeoData* geodata;
		Quadtree<double,GeoData>* quadtree;
		
	 public:
			
		/** Constructor for the position Manager
		 *
		 */
		PositionManager(GeoData* _geodata, jafar::lgl::Quadtree<double,GeoData>* _qtree);

		~PositionManager();

		bool isAtCenter(const NavLocation& loc) const;

		// check if a position is at the edge or not of the a raster cell
		bool isAtEdge(const NavLocation& loc) const;
			
		// check if a position is at the border of the cell
		bool isAtBorder(const NavLocation& loc) const;				

		/** Get the enter positions to the nav graph of a given position inside
		 * the position map with the decomposition decomp. if this position is
		 * already inside the graph then do nothing
		 */
		bool getGEnterPos(const NavLocation& loc, std::vector<NavLocation>& enterLocs) const;
			
		// get adjacents cells to a given position
		bool getAdjRasterCells(const NavLocation& loc, std::vector<RasterCellIndex>& cells) const;

		// get adjacents cluster to a given position
		bool getAdjClusters(const NavLocation& loc, std::vector<RasterCellIndex>& clusters) const;

		bool getTopLeftRasterCell(const NavLocation& loc, RasterCellIndex& cell);
		
		bool getTopLeftCluster(const NavLocation& loc, RasterCellIndex& cell);

		bool inSameCluster(const NavLocation& loc1, const NavLocation& loc2, RasterCellIndex& clusterRoot);

		/** Return the cluster in which lays the edge between loc1 and loc2
		 *
		 * This cluster is represented by a rectangle zone on which borders lay
		 * the two locations. 
		 *
		 * @param loc1 Source position of the edge we are looking for
		 * @param loc2 Target position of the edge we are looking for
		 * @param cluster output of this function, the corresponding raster rectangle
		 *
		 */
		bool getCluster(const NavLocation& loc1, const NavLocation& loc2, RasterRect& cluster);

		/** Return the decomp cluster in which lays the edge between loc1 and loc2
		 *
		 * This cluster is a leaf of the quadtree decomposition structure.
		 * Data can be updated directly inside.
		 *
		 * @param loc1 Source position of the edge we are looking for
		 * @param loc2 Target position of the edge we are looking for
		 * @param cluster output of this function, the pointer to the corresponding DecompCluster
		 *
		 */
		bool getCluster(const NavLocation& loc1, const NavLocation& loc2, DecompCluster** cluster);
		bool getCluster(const RasterCellIndex& clusterRoot, DecompCluster** cluster);

		/** Returns the decomposition cluster for one location
		 *
		 * This function should not be used to try to build edges.
		 * It is only intended for information pulling.
		 * In the returned decomposition cluster the user can find the
		 * associated terrain state but it is not garanteed on which side of
		 * the location the information is pulled as it depends on how the land
		 * decomposition
		 */
		bool getClusterUnordered(const NavLocation& loc, DecompCluster** cluster);

	 };


  }
}

#endif /* LGL_POSITION_MANAGER_HPP */
