/**
 * \file RasterRect.cpp
 *
 * Source file defining the RasterRect object
 *
 * A RasterRect representes a group
 * of pixels in a cell of a quadtree.
 *
 * \date 2009-09-05
 * \author Redouane Boumghar, Hung Cao
 *
 * \ingroup lgl
 *
 **/

#include <lgl/Raster.hpp>

using namespace jafar::lgl;

/*****************************************************************************
 * RasterRect member functions definition
 *****************************************************************************/

// Default constructor
RasterRect::RasterRect(): 
  xtopleft(0),
  ytopleft(0),
  xbottomright(-1),
  ybottomright(-1), 
  xpsize(0),
  ypsize(0)
{
	// simple setting of default values 
}

// Parametrized constructor
RasterRect::RasterRect( int _xroot, int _yroot, int _xsize, int _ysize )
{
	// Top left corner coordinates
	xtopleft = _xroot;
	ytopleft = _yroot;
	// Size information
	xpsize = _xsize;
	ypsize = _ysize;
	// Top-left pixel's corner coordinates of the bottom right pixel
	xbottomright = _xroot + _xsize - 1;		
	ybottomright = _yroot + _ysize - 1;
}


// Valid rectangles can have size Zero. We are checking if the bottom right
// corner is really at the bottom right.
bool RasterRect::isValid() const
{ return ytopleft <= ybottomright && xtopleft <= xbottomright; }

//-------------------- Getters -------------------/

int &RasterRect::rLeft()
{ return xtopleft; }

int & RasterRect::rTop()
{ return ytopleft; }

int & RasterRect::rRight()
{ return xbottomright; }

int & RasterRect::rBottom()
{ return ybottomright; }

int RasterRect::xroot() const
{ return xtopleft; }

int RasterRect::yroot() const
{ return ytopleft; }

int RasterRect::xend() const
{ return xbottomright; }

int RasterRect::yend() const
{ return ybottomright; }

/*
int RasterRect::ysize() const
{ return  ypsize;}

int RasterRect::xsize() const
{ return  xpsize; }
*/

RasterPoint RasterRect::topLeft() const
{ return RasterPoint(xtopleft, ytopleft); }

RasterPoint RasterRect::bottomRight() const
{ return RasterPoint(xbottomright, ybottomright); }

RasterPoint RasterRect::topRight() const
{ return RasterPoint(xbottomright, ytopleft); }

RasterPoint RasterRect::bottomLeft() const
{ return RasterPoint(xtopleft, ybottomright); }

RasterPoint RasterRect::center() const
{ return RasterPoint((xtopleft+xbottomright)/2, (ytopleft+ybottomright)/2); }

//-------------------- Setters -------------------//

void RasterRect::setXTopLeft( int pos )
{ 
	xtopleft = pos; 
	// Adjust size parameter accordingly
	xpsize = xbottomright - xtopleft + 1;
}

void RasterRect::setYTopLeft( int pos )
{ 
	ytopleft = pos;
	// Adjust size parameter accordingly
	ypsize = ybottomright - ytopleft + 1;
}

void RasterRect::setXBottomRight( int pos )
{ 
	xbottomright = pos;
	// Adjust size parameter accordingly
	xpsize = xbottomright - xtopleft + 1;
}

void RasterRect::setYBottomRight( int pos )
{ 
	ybottomright = pos;
	// Adjust size parameter accordingly
	ypsize = ybottomright - ytopleft + 1;
}

void RasterRect::setX( int x )
{ setXTopLeft(x); }

void RasterRect::setY( int y )
{ setYTopLeft(y); }


//-------------------- Pullers -------------------//

void RasterRect::rect( int& _xtopleft, int& _ytopleft, int& _xsize, int& _ysize ) const {
	_xtopleft = xtopleft;
	_ytopleft = ytopleft;
	_xsize = xpsize;
	_ysize = ypsize;
}

void RasterRect::coords( int& _xtopleft, int& _ytopleft, int& _xbottomright, int& _ybottomright ) const {
	_xtopleft = xtopleft;
	_ytopleft = ytopleft;
	_xbottomright = xbottomright;
	_ybottomright = ybottomright;
}

int RasterRect::size() const
{ return (xpsize*ypsize); }

bool RasterRect::contains( const RasterCellIndex& cell, bool inside) const {
	if ( inside ) {
		// strict container in the manner of topleft raster coordinates
		return (cell.i >= xtopleft) && (cell.i <= xbottomright) &&
		       (cell.j >= ytopleft) && (cell.j <= ybottomright);
	} else {
		// With this behaviour even cells only touching the rectangle on its
		// right and bottom are considered inside.
		return (cell.i >= xtopleft) && (cell.i <= (xbottomright+1)) &&
		       (cell.j >= ytopleft) && (cell.j <= (ybottomright+1));
	}
}

bool RasterRect::contains( const RasterRect& rect, bool inside ) const {
	if (inside)
	{
		return (     (xtopleft <= rect.xtopleft) && (ytopleft <= rect.ytopleft)
		 && (xbottomright >= rect.xbottomright) && (ybottomright >= rect.ybottomright) );
	} else {
		return (    (xtopleft <= rect.xtopleft) && (ytopleft <= rect.ytopleft)
		 && ((xbottomright+1) >= rect.xbottomright) && ((ybottomright+1) >= rect.ybottomright) );
	}
}

void RasterRect::setCoords( int _xtopleft, int _ytopleft, int _xbottomright, int _ybottomright ) {
	xtopleft=_xtopleft;
	ytopleft=_ytopleft;
	xbottomright=_xbottomright;
	ybottomright=_ybottomright;
}

void RasterRect::setRect(  int _xroot, int _yroot, int _xsize, int _ysize ) {
	xtopleft = _xroot;
	ytopleft = _yroot;
	xbottomright = _xroot + _xsize - 1;		
	ybottomright = _yroot + _ysize - 1;
	xpsize = _xsize;
	ypsize = _ysize;
}

bool RasterRect::intersects(const RasterRect &_rect,  RasterRect &result) const {
  // Strict falsers
  if(xbottomright < _rect.xtopleft) return false;
  if(xtopleft > _rect.xbottomright) return false;
  if(ybottomright < _rect.ytopleft) return false;
  if(ytopleft > _rect.ybottomright) return false;

  // Get the resulting rectangle
  result.setCoords(std::max(xtopleft, _rect.xtopleft), 
						 std::max(ytopleft, _rect.ytopleft),
						 std::min(xbottomright, _rect.xbottomright), 
						 std::min(ybottomright, _rect.ybottomright));
		
  return true;
}


///////////////////////// OPERATORS /////////////////////////

RasterRect&  RasterRect::operator=(const RasterRect &rect) {
	xtopleft = rect.xtopleft;
	ytopleft = rect.ytopleft;
	xbottomright = rect.xbottomright;		
	ybottomright = rect.ybottomright;		
	xpsize = rect.xpsize;
	ypsize = rect.ypsize;
	return *this;
}

std::ostream& jafar::lgl::operator<<(std::ostream& out, const RasterRect &rect) {
  out << "topleft (" << rect.xtopleft << ", " << rect.ytopleft << "), size (" << rect.xsize() << ", "<< rect.ysize() << ")";
  return out;
}

bool jafar::lgl::operator==( const RasterRect &rect1, const RasterRect &rect2 ) {
  return ((rect1.xtopleft==rect2.xtopleft)
			 && (rect1.ytopleft==rect2.ytopleft)
			 && (rect1.xbottomright==rect2.xbottomright)
			 && (rect1.ybottomright==rect2.ybottomright));	 
}

bool jafar::lgl::operator!=( const RasterRect &rect1, const RasterRect &rect2 ) {
  return ((rect1.xtopleft!=rect2.xtopleft)
			 || (rect1.ytopleft!=rect2.ytopleft)
			 || (rect1.xbottomright!=rect2.xbottomright)
			 || (rect1.ybottomright!=rect2.ybottomright));	 
}

